/****************************************************************************
 *
 *      $Id: pager.c,v 1.4 2003/08/06 22:52:04 benjl Exp $
 *
 *      Description: Device driver glue for SOS project.
 *
 *      Author:      Ben Leslie
 *
 ****************************************************************************/

#ifndef __DEV_H_
#define __DEV_H_

#include <l4.h>
#include <stdint.h>

typedef enum direction { mem_to_bus, bus_to_mem, consistent } direction_t;

typedef void* bus_space_t;

typedef struct {
	unsigned int c;
	bus_space_t *v;
} bs_seq;

typedef void* dma_handle_t;
typedef unsigned long dma_addr_t;
typedef void* bus_addr_t;
typedef void* host_handle_t;
typedef void* mem_handle_t;

typedef struct {
	void (*function) (void *);
	void *data;
} exec_handle_t;

typedef struct {
    mem_handle_t handle;
    unsigned long length;
    uintptr_t key;
} buf_t;

typedef struct {
    unsigned int c;
    buf_t *v;
} packet_t; 

typedef struct {
	unsigned int c;
	packet_t * v;
} pkt_seq;


#define new_mem_handle(x) x

#define EXECUTE_HANDLE(x) (x.function(x.data))

#define NULL_EXEC_HANDLE { NULL, NULL }

#define get_addr(x) (x)
mem_handle_t map_physical(host_handle_t host, void *phys_addr, uintptr_t size);
int register_cpu_interrupt_handler(host_handle_t host, int num, 
				   void *fn, void *data);
int get_pinnable_memory(uintptr_t size, mem_handle_t * handle);
dma_addr_t setup_dma(dma_handle_t dma, mem_handle_t handle, uintptr_t size,
		     direction_t dir);
void dev_sleep(int micros);
void dev_yield(void);

void irq_handler(L4_ThreadId_t tid, L4_MsgTag_t tag, L4_Msg_t *msg);

#endif
