/*-
 * Copyright (c) 2003 Cristan Szmajda.	All rights reserved. 
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *	notice, this list of conditions, and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright
 *	notice, this list of conditions, and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution. 
 * 3. All advertising materials mentioning features or use of this software
 *	must display the following acknowledgement:
 * 	This product includes software developed by Cristan Szmajda. 
 * 4. The name Cristan Szmajda may not be used to endorse or promote
 *	products derived from this software without specific prior written
 *	permission. 
 * 
 * THIS SOFTWARE IS PROVIDED BY CRISTAN SZMAJDA ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL CRISTAN SZMAJDA BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 * 
 * $Id: inttypes.h,v 1.1 2003/07/28 05:24:40 benjl Exp $
 */

#ifndef _INTTYPES_H_
#define	_INTTYPES_H_

#include <stddef.h>
#include <stdint.h>

typedef	struct { intmax_t quot, rem; } imaxdiv_t;

intmax_t imaxabs(intmax_t i);
imaxdiv_t imaxdiv(intmax_t num, intmax_t denom);
intmax_t strtoimax(const char *restrict s, char **restrict endp, int base);
uintmax_t strtoumax(const char *restrict s, char **restrict endp, int base);
intmax_t wcstoimax(const wchar_t *restrict s, wchar_t **restrict endp, int base);
uintmax_t wcstoumax(const wchar_t *restrict s, wchar_t **restrict endp, int base);

/* printf conversion specifiers */
#define	PRIdMAX		"lld"		/* intmax_t */
#define	PRIiMAX		"lli"		/* intmax_t */
#define	PRIoMAX		"llo"		/* uintmax_t */
#define	PRIuMAX		"llu"		/* uintmax_t */
#define	PRIxMAX		"llx"		/* uintmax_t */
#define	PRIXMAX		"llX"		/* uintmax_t */
#define	PRIdPTR		"ld"		/* intptr_t */
#define	PRIiPTR		"li"		/* intptr_t */
#define	PRIoPTR		"lo"		/* uintptr_t */
#define	PRIuPTR		"lu"		/* uintptr_t */
#define	PRIxPTR		"lx"		/* uintptr_t */
#define	PRIXPTR		"lX"		/* uintptr_t */
#define	PRId8		"hhd"		/* int8_t */
#define	PRIi8		"hhi"		/* int8_t */
#define	PRIo8		"hho"		/* uint8_t */
#define	PRIu8		"hhu"		/* uint8_t */
#define	PRIx8		"hhx"		/* uint8_t */
#define	PRIX8		"hhX"		/* uint8_t */
#define	PRId16		"hd"		/* int16_t */
#define	PRIi16		"hi"		/* int16_t */
#define	PRIo16		"ho"		/* uint16_t */
#define	PRIu16		"hu"		/* uint16_t */
#define	PRIx16		"hx"		/* uint16_t */
#define	PRIX16		"hX"		/* uint16_t */
#define	PRId32		"d"		/* int32_t */
#define	PRIi32		"i"		/* int32_t */
#define	PRIo32		"o"		/* uint32_t */
#define	PRIu32		"u"		/* uint32_t */
#define	PRIx32		"x"		/* uint32_t */
#define	PRIX32		"X"		/* uint32_t */
#define	PRId64		"lld"		/* int64_t */
#define	PRIi64		"lli"		/* int64_t */
#define	PRIo64		"llo"		/* uint64_t */
#define	PRIu64		"llu"		/* uint64_t */
#define	PRIx64		"llx"		/* uint64_t */
#define	PRIX64		"llX"		/* uint64_t */
#define	PRIdLEAST8	PRId8		/* int_least8_t */
#define	PRIiLEAST8	PRIi8		/* int_least8_t */
#define	PRIoLEAST8	PRIo8		/* uint_least8_t */
#define	PRIuLEAST8	PRIu8		/* uint_least8_t */
#define	PRIxLEAST8	PRIx8		/* uint_least8_t */
#define	PRIXLEAST8	PRIX8		/* uint_least8_t */
#define	PRIdLEAST16	PRId16		/* int_least16_t */
#define	PRIiLEAST16	PRIi16		/* int_least16_t */
#define	PRIoLEAST16	PRIo16		/* uint_least16_t */
#define	PRIuLEAST16	PRIu16		/* uint_least16_t */
#define	PRIxLEAST16	PRIx16		/* uint_least16_t */
#define	PRIXLEAST16	PRIX16		/* uint_least16_t */
#define	PRIdLEAST32	PRId32		/* int_least32_t */
#define	PRIiLEAST32	PRIi32		/* int_least32_t */
#define	PRIoLEAST32	PRIo32		/* uint_least32_t */
#define	PRIuLEAST32	PRIu32		/* uint_least32_t */
#define	PRIxLEAST32	PRIx32		/* uint_least32_t */
#define	PRIXLEAST32	PRIX32		/* uint_least32_t */
#define	PRIdLEAST64	PRId64		/* int_least64_t */
#define	PRIiLEAST64	PRIi64		/* int_least64_t */
#define	PRIoLEAST64	PRIo64		/* uint_least64_t */
#define	PRIuLEAST64	PRIu64		/* uint_least64_t */
#define	PRIxLEAST64	PRIx64		/* uint_least64_t */
#define	PRIXLEAST64	PRIX64		/* uint_least64_t */
#define	PRIdFAST8	PRId8		/* int_fast8_t */
#define	PRIiFAST8	PRIi8		/* int_fast8_t */
#define	PRIoFAST8	PRIo8		/* uint_fast8_t */
#define	PRIuFAST8	PRIu8		/* uint_fast8_t */
#define	PRIxFAST8	PRIx8		/* uint_fast8_t */
#define	PRIXFAST8	PRIX8		/* uint_fast8_t */
#define	PRIdFAST16	PRId16		/* int_fast16_t */
#define	PRIiFAST16	PRIi16		/* int_fast16_t */
#define	PRIoFAST16	PRIo16		/* uint_fast16_t */
#define	PRIuFAST16	PRIu16		/* uint_fast16_t */
#define	PRIxFAST16	PRIx16		/* uint_fast16_t */
#define	PRIXFAST16	PRIX16		/* uint_fast16_t */
#define	PRIdFAST32	PRId32		/* int_fast32_t */
#define	PRIiFAST32	PRIi32		/* int_fast32_t */
#define	PRIoFAST32	PRIo32		/* uint_fast32_t */
#define	PRIuFAST32	PRIu32		/* uint_fast32_t */
#define	PRIxFAST32	PRIx32		/* uint_fast32_t */
#define	PRIXFAST32	PRIX32		/* uint_fast32_t */
#define	PRIdFAST64	PRId64		/* int_fast64_t */
#define	PRIiFAST64	PRIi64		/* int_fast64_t */
#define	PRIoFAST64	PRIo64		/* uint_fast64_t */
#define	PRIuFAST64	PRIu64		/* uint_fast64_t */
#define	PRIxFAST64	PRIx64		/* uint_fast64_t */
#define	PRIXFAST64	PRIX64		/* uint_fast64_t */

/* scanf conversion specifiers */
#define	SCNdMAX		PRIdMAX		/* intmax_t */
#define	SCNiMAX		PRIiMAX		/* intmax_t */
#define	SCNoMAX		PRIoMAX		/* uintmax_t */
#define	SCNuMAX		PRIuMAX		/* uintmax_t */
#define	SCNxMAX		PRIxMAX		/* uintmax_t */
#define	SCNXMAX		PRIXMAX		/* uintmax_t */
#define	SCNdPTR		PRIdPTR		/* intptr_t */
#define	SCNiPTR		PRIiPTR		/* intptr_t */
#define	SCNoPTR		PRIoPTR		/* uintptr_t */
#define	SCNuPTR		PRIuPTR		/* uintptr_t */
#define	SCNxPTR		PRIxPTR		/* uintptr_t */
#define	SCNXPTR		PRIXPTR		/* uintptr_t */
#define	SCNd8		PRId8		/* int8_t */
#define	SCNi8		PRIi8		/* int8_t */
#define	SCNo8		PRIo8		/* uint8_t */
#define	SCNu8		PRIu8		/* uint8_t */
#define	SCNx8		PRIx8		/* uint8_t */
#define	SCNX8		PRIX8		/* uint8_t */
#define	SCNd16		PRId16		/* int16_t */
#define	SCNi16		PRIi16		/* int16_t */
#define	SCNo16		PRIo16		/* uint16_t */
#define	SCNu16		PRIu16		/* uint16_t */
#define	SCNx16		PRIx16		/* uint16_t */
#define	SCNX16		PRIX16		/* uint16_t */
#define	SCNd32		PRId32		/* int32_t */
#define	SCNi32		PRIi32		/* int32_t */
#define	SCNo32		PRIo32		/* uint32_t */
#define	SCNu32		PRIu32		/* uint32_t */
#define	SCNx32		PRIx32		/* uint32_t */
#define	SCNX32		PRIX32		/* uint32_t */
#define	SCNd64		PRId64		/* int64_t */
#define	SCNi64		PRIi64		/* int64_t */
#define	SCNo64		PRIo64		/* uint64_t */
#define	SCNu64		PRIu64		/* uint64_t */
#define	SCNx64		PRIx64		/* uint64_t */
#define	SCNX64		PRIX64		/* uint64_t */
#define	SCNdLEAST8	PRIdLEAST8	/* int_least8_t */
#define	SCNiLEAST8	PRIiLEAST8	/* int_least8_t */
#define	SCNoLEAST8	PRIoLEAST8	/* uint_least8_t */
#define	SCNuLEAST8	PRIuLEAST8	/* uint_least8_t */
#define	SCNxLEAST8	PRIxLEAST8	/* uint_least8_t */
#define	SCNXLEAST8	PRIXLEAST8	/* uint_least8_t */
#define	SCNdLEAST16	PRIdLEAST16	/* int_least16_t */
#define	SCNiLEAST16	PRIiLEAST16	/* int_least16_t */
#define	SCNoLEAST16	PRIoLEAST16	/* uint_least16_t */
#define	SCNuLEAST16	PRIuLEAST16	/* uint_least16_t */
#define	SCNxLEAST16	PRIxLEAST16	/* uint_least16_t */
#define	SCNXLEAST16	PRIXLEAST16	/* uint_least16_t */
#define	SCNdLEAST32	PRIdLEAST32	/* int_least32_t */
#define	SCNiLEAST32	PRIiLEAST32	/* int_least32_t */
#define	SCNoLEAST32	PRIoLEAST32	/* uint_least32_t */
#define	SCNuLEAST32	PRIuLEAST32	/* uint_least32_t */
#define	SCNxLEAST32	PRIxLEAST32	/* uint_least32_t */
#define	SCNXLEAST32	PRIXLEAST32	/* uint_least32_t */
#define	SCNdLEAST64	PRIdLEAST64	/* int_least64_t */
#define	SCNiLEAST64	PRIiLEAST64	/* int_least64_t */
#define	SCNoLEAST64	PRIoLEAST64	/* uint_least64_t */
#define	SCNuLEAST64	PRIuLEAST64	/* uint_least64_t */
#define	SCNxLEAST64	PRIxLEAST64	/* uint_least64_t */
#define	SCNXLEAST64	PRIXLEAST64	/* uint_least64_t */
#define	SCNdFAST8	PRIdFAST8	/* int_fast8_t */
#define	SCNiFAST8	PRIiFAST8	/* int_fast8_t */
#define	SCNoFAST8	PRIoFAST8	/* uint_fast8_t */
#define	SCNuFAST8	PRIuFAST8	/* uint_fast8_t */
#define	SCNxFAST8	PRIxFAST8	/* uint_fast8_t */
#define	SCNXFAST8	PRIXFAST8	/* uint_fast8_t */
#define	SCNdFAST16	PRIdFAST16	/* int_fast16_t */
#define	SCNiFAST16	PRIiFAST16	/* int_fast16_t */
#define	SCNoFAST16	PRIoFAST16	/* uint_fast16_t */
#define	SCNuFAST16	PRIuFAST16	/* uint_fast16_t */
#define	SCNxFAST16	PRIxFAST16	/* uint_fast16_t */
#define	SCNXFAST16	PRIXFAST16	/* uint_fast16_t */
#define	SCNdFAST32	PRIdFAST32	/* int_fast32_t */
#define	SCNiFAST32	PRIiFAST32	/* int_fast32_t */
#define	SCNoFAST32	PRIoFAST32	/* uint_fast32_t */
#define	SCNuFAST32	PRIuFAST32	/* uint_fast32_t */
#define	SCNxFAST32	PRIxFAST32	/* uint_fast32_t */
#define	SCNXFAST32	PRIXFAST32	/* uint_fast32_t */
#define	SCNdFAST64	PRIdFAST64	/* int_fast64_t */
#define	SCNiFAST64	PRIiFAST64	/* int_fast64_t */
#define	SCNoFAST64	PRIoFAST64	/* uint_fast64_t */
#define	SCNuFAST64	PRIuFAST64	/* uint_fast64_t */
#define	SCNxFAST64	PRIxFAST64	/* uint_fast64_t */
#define	SCNXFAST64	PRIXFAST64	/* uint_fast64_t */

#endif	/* _INTTYPES_H_ */
