/****************************************************************************
 *
 *      $Id: network.c,v 1.1 2003/09/10 11:44:38 benjl Exp $
 *
 *      Description: Initialise the network stack and NFS library.
 *
 *      Author:      Ben Leslie
 *
 ****************************************************************************/

#include <string.h>
#include <tulip.h>
#include <lwip/mem.h>
#include <lwip/memp.h>
#include <lwip/udp.h>
#include <netif/etharp.h>
#include <nfs.h>
#include <netif/sosif.h>

#include "hardware.h"
#include "frames.h"

struct cookie mnt_point;

/* This is the directory on the NFS server (asyst09) that will be mounted
 * for your file system. Feel free to create a subdirectory for yourself/group
 * to put your files in. 
 * Watch out for swap file collisions with your partner! :)
 */
#define NFS_DIR "/tftpboot"

void
network_init(void)
{
	struct ip_addr ipaddr, netmask, gw;
	struct ip_addr nfs_server;
	struct netif *netif;
	int i, r;

	l4e_printf("Initialising lwip\n");

	/* Initialise lwIP */
	mem_init();
	memp_init();
	pbuf_init();
	netif_init();
	udp_init();
	etharp_init();

	/* This is setup for the MIPS box. If you are using 
	   sulima then you should user the 192.168.2.0 network */
	IP4_ADDR(&gw, 192,168,1,1);

	/* FIXME: Make sure this IP is right for your group!
	 * PMON ip addresses for the machines are of the form:
	 *          192.168.1.<box number>
	 *
	 * Because you will be most likely changing machines, you can use 
	 * static addresses of the form:
	 *          192.168.1.1<aosgroup>
	 * and
	 *          192.168.1.2<aosgroup>
	 *
	 * This allows you to have an IP per partner in your group.
	 * One IP for group 2 would be 192.168.1.102, group 12 would have
	 * 192.168.1.112, etc.
	 *
	 * If you do not have a partner/group number, or you would like
	 * more IP addresses for some reason, email the class account.
	 */
	assert(!"Fix the IP address and read the big comment here");
	IP4_ADDR(&ipaddr, 192,168,1,100);

	IP4_ADDR(&netmask, 255,255,255,0);

	/* Setup the network interface */
	netif = netif_add(&ipaddr, &netmask, &gw,
			  sosif_init, ip_input);
	netif_set_default(netif);
	netif->state = tulip;
	tulip_get_macaddr(tulip, netif->hwaddr);

	/* Provide the network card with some recieve descriptors */
	for(i=0; i < 8; i++) {
		uint32_t frame = frame_alloc();
		tulip_freepacket(tulip, frame, 1536, 0);
	}

	tulip_register_callback(tulip, NULL, sosif_tx_freepacket, 
				low_level_input);

	/* Enable recieving packets */
	tulip_enable_receive(tulip);

	/* Initialise NFS */
	IP4_ADDR(&nfs_server, 192,168,1,1);

	r = nfs_init(nfs_server);

	assert(r == 0);

	/* Print out the exports on this server */
	mnt_get_export_list();

	/* Mount aos_nfs */
	r = mnt_mount( NFS_DIR, &mnt_point);
	if( r != 0 )
		l4e_printf( "Error mounting path '" NFS_DIR "'!\n" );
	else
		l4e_printf( "Successfully mounted '" NFS_DIR"'\n" );
}
